'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var client = require('react-dom/client');
var reactDom = require('react-dom');
var reflection_js = require('react-reconciler/reflection.js');
var TestWrapper = require('./TestWrapper.js');
var element = require('./element.js');
var compat = require('./compat.js');
var types = require('./types.js');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

const connected = new Set();
class Root {
  get props() {
    return this.withRoot(root => root.props);
  }
  get isDOM() {
    return this.withRoot(root => root.isDOM);
  }
  get type() {
    return this.withRoot(root => root.type);
  }
  get instance() {
    return this.withRoot(root => root.instance);
  }
  get children() {
    return this.withRoot(root => root.children);
  }
  get descendants() {
    return this.withRoot(root => root.descendants);
  }
  get domNodes() {
    return this.withRoot(root => root.domNodes);
  }
  get domNode() {
    return this.withRoot(root => root.domNode);
  }
  get mounted() {
    return this.wrapper != null && this.wrapper.rootRef != null;
  }
  constructor(tree, {
    render = defaultRender,
    resolveRoot = defaultResolveRoot
  } = {}) {
    this.wrapper = null;
    this.element = document.createElement('div');
    this.reactRoot = null;
    this.root = null;
    this.acting = false;
    this.destroyed = false;
    this.actCallbacks = new Set();
    this.tree = tree;
    this.render = render;
    this.resolveRoot = resolveRoot;
    this.mount();
  }
  act(action, {
    update = true
  } = {}) {
    if (this.destroyed) {
      // eslint-disable-next-line no-console
      console.error('Warning: attempting to perform an act on a destroyed root. This can lead to state changes not being applied');
    }
    const updateWrapper = update ? this.update.bind(this) : noop;
    let result;
    if (this.acting) {
      result = action();
      updateWrapper();
      return result;
    }
    this.acting = true;

    /* act has two versions, act(() => void) or await act(async () => void)
     * The async version will wrap the inner async function so that it maintains an act
     * scope while it completes it's work asynchronously, and flushes updates to the
     * wrapper at the end.
     *
     * The non-async version will invoke the inner function synchronously and will flush
     * updates synchronously at the end.
     *
     * Typically the developer invokes act directly and knows which of these to use, but
     * since this is a generic act within Root it may get called with _either_ a synchronous
     * or asynchronous action. As a result, we can't naively await it, and instead have to
     * check the return value of result to determine if it needs awaiting so that it can
     * properly flush updates.
     */
    const possiblyAwaitableAct = compat.act(() => {
      reactDom.flushSync(() => {
        result = action();
      });
      if (isPromise(result)) {
        if (this.destroyed) {
          return result;
        }

        /* return a thenable which when invoked will add react's callback (to clear queue)
         * into a set. If the result ever resolves we will remove the callback from the Set.
         * If it doesn't we will call all callbacks in the Set when this root is destroyed which
         * will unblock any stuck queues allowing subsequent tests to run
         * Note: This can be cleanly achieved with a `Promise.race` but that adds an extra micro-task
         * which can potentially break fragile logic that depend on specific update timings.
         * (eg. testing errors with useLazyQuery from apollo )
         */

        return {
          then: (callback, error) => {
            this.actCallbacks.add(callback);
            if (this.destroyed) {
              return Promise.resolve();
            }
            return result.then(value => {
              this.actCallbacks.delete(callback);
              if (!this.destroyed) {
                return callback(value);
              }
            }, value => {
              this.actCallbacks.delete(callback);
              if (!this.destroyed) {
                return error(value);
              }
            });
          }
        };
      }
      return undefined;
    });
    updateWrapper();
    if (isPromise(result)) {
      /* If the result has returned a promise, then we must first await the thenable object that
       * act returned (in order to clear the act scope and flush promises). Second, we must await
       * the result itself which is the purpose of this function - to perform act and return the
       * underlying result. Third, we must update the wrapper itself so that it remains consistent.
       */
      const getResultAsync = async () => {
        await possiblyAwaitableAct;
        const resolvedResultValue = await result;
        updateWrapper();
        this.acting = false;
        return resolvedResultValue;
      };
      return getResultAsync();
    } else {
      /* If the result didn't return a promise, then the synchronous behaviour has already completed
       * and the synchronous version of act has too. Therefore it is safe to return the result directly.
       */
      this.acting = false;
      return result;
    }
  }
  html() {
    return this.withRoot(root => root.html());
  }
  text() {
    return this.withRoot(root => root.text());
  }
  is(type) {
    return this.withRoot(root => root.is(type));
  }
  prop(key) {
    return this.withRoot(root => root.prop(key));
  }
  data(key) {
    return this.withRoot(root => root.data(key));
  }
  find(type, props) {
    return this.withRoot(root => root.find(type, props));
  }
  findAll(type, props) {
    return this.withRoot(root => root.findAll(type, props));
  }
  findWhere(predicate) {
    return this.withRoot(root => root.findWhere(predicate));
  }
  findAllWhere(predicate) {
    return this.withRoot(root => root.findAllWhere(predicate));
  }
  trigger(prop, ...args) {
    return this.withRoot(root => root.trigger(prop, ...args));
  }
  triggerKeypath(...args) {
    return this.withRoot(root => root.triggerKeypath(...args));
  }
  mount() {
    if (this.mounted) {
      throw new Error('Attempted to mount a node that was already mounted');
    }
    if (this.element.parentNode == null) {
      document.body.appendChild(this.element);
      connected.add(this);
    }
    this.reactRoot = client.createRoot(this.element);
    this.act(() => {
      this.reactRoot.render( /*#__PURE__*/React__default["default"].createElement(TestWrapper.TestWrapper, {
        render: this.render,
        ref: wrapper => {
          this.wrapper = wrapper;
        }
      }, this.tree));
    });
  }
  unmount() {
    if (!this.mounted) {
      throw new Error('You attempted to unmount a node that was already unmounted');
    }
    this.ensureRoot();
    return this.act(() => this.reactRoot.unmount());
  }
  async destroy() {
    const {
      element,
      mounted
    } = this;
    let mountedPromise;
    if (mounted) {
      mountedPromise = this.unmount();
    }
    element.remove();
    connected.delete(this);
    this.destroyed = true;
    await mountedPromise;
    this.actCallbacks.forEach(callback => callback());
  }
  setProps(props) {
    this.ensureRoot();
    this.act(() => this.wrapper.setProps(props));
  }
  forceUpdate() {
    this.ensureRoot();
    this.act(() => this.wrapper.forceUpdate());
  }
  debug(options) {
    this.ensureRoot();
    return this.root.debug(options);
  }
  toString() {
    return this.withRoot(root => root.toString());
  }
  update() {
    if (this.wrapper == null) {
      this.root = null;
    } else if (this.mounted) {
      const rootFiber = this.wrapper.rootRef._reactInternals;
      const topElement = fiberToElement(reflection_js.findCurrentFiberUsingSlowPath(rootFiber), this);
      this.root = this.resolveRoot(topElement);
    }
  }
  ensureRoot() {
    if (!this.mounted || this.root == null) {
      throw new Error('Attempted to operate on a mounted tree, but the component is no longer mounted');
    }
  }
  withRoot(withRoot) {
    this.ensureRoot();
    return withRoot(this.root);
  }
}
function defaultResolveRoot(element) {
  return element.children[0];
}
function defaultRender(element) {
  return element;
}
function fiberToElement(node, root) {
  if (node.tag === types.Tag.HostText) {
    return node.memoizedProps;
  }
  const props = node.memoizedProps;
  const children = childrenToTree(node.child, root);
  return new element.Element({
    tag: node.tag,
    type: node.type,
    props,
    instance: node.stateNode
  }, children, root);
}
function childrenToTree(fiber, root) {
  let currentFiber = fiber;
  const children = [];
  while (currentFiber != null) {
    const result = fiberToElement(currentFiber, root);
    children.push(result);
    currentFiber = currentFiber.sibling;
  }
  return children;
}
function isPromise(value) {
  return value != null && typeof value === 'object' && typeof value.then === 'function';
}
function noop() {}

exports.Root = Root;
exports.connected = connected;
